"""
@author: luojunhui
"""
import time

from uuid import uuid4

from applications.functions.log import logging
from applications.static.config import db_article


class SearchDeal2(object):
    """
    搜索接口处理逻辑
    """

    def __init__(self, params, mysql_client):
        self.content_id = None
        self.account_name = None
        self.contents = None
        self.title = None
        self.gh_id = None
        self.params = params
        self.mysql_client = mysql_client
        self.trace_id = "search-{}-{}".format(str(uuid4()), str(int(time.time())))

    def check_params(self):
        """
        检查请求params
        :return:
        """
        try:
            self.gh_id = self.params['ghId']
            self.title = self.params['title'].split("@@")[-1].replace("'", "")
            self.contents = self.params['content'].replace("'", "")
            self.account_name = self.params['accountName'].replace("'", "")
            self.content_id = self.params['articleId']
            logging(
                code="1001",
                info="搜索视频内容接口请求成功, 参数校验成功",
                port="title_to_search",
                trace_id=self.trace_id,
                data=self.params
            )
            return None
        except Exception as e:
            result = {
                "status": "fail",
                "code": 1,
                "message": str(e),
                "info": "params check error"
            }
            logging(
                code="4001",
                info="搜索视频内容接口请求成功, 参数校验失败",
                port="title_to_search",
                trace_id=self.trace_id,
                data=self.params
            )
            return result

    async def input_queue(self):
        """
        把数据插入待处理队列
        :return:
        """
        request_time = int(time.time())
        insert_sql = f"""
                        INSERT INTO {db_article}
                            (trace_id, content_id, gh_id, account_name, article_title, article_text, content_status, success, request_time_stamp)
                        VALUES 
                            (%s, %s, %s, %s, %s, %s, %s, %s, %s);"""
        await self.mysql_client.async_insert(
            sql=insert_sql,
            params=(self.trace_id, self.content_id, self.gh_id, self.account_name, self.title, self.contents, 0, 0, request_time)
        )
        logging(
            code="1002",
            info="成功记录请求数据到mysql中",
            trace_id=self.trace_id
        )

    async def deal(self):
        """
        deal
        :return:
        """
        params_error = self.check_params()
        if params_error:
            return params_error
        else:
            # 记录
            await self.input_queue()
            res = {
                "status": "success input to article queue",
                "code": 0,
                "traceId": self.trace_id
            }
            return res
